""" A module to read a DART dump file.
"""

import os, sys, re

#from Euclid import Euclid_TC_codes, Euclid_TM_codes
import Euclid_TC_codes, Euclid_TM_codes

if sys.version_info[0] < 3:
   from ConfigParser import RawConfigParser
else:
   from configparser import RawConfigParser


""" This class provides an enum like constant for decoding the type of line read from the file. """
class LineType:
    TM, TC, Event, TM_PACKET, Empty, Err = range(6)

#logTime = ''

""" This class implements the Dump file reader. """
class EchoReaderTwo(object):


    def __init__(self,config_type):
        super(EchoReaderTwo, self).__init__()

        self._filepath = ""
        #self._fileId = 0
        self._fileId = None
        self._line = ""
        self._error = 0
        self._eof = 0
        self._stop = 0
        self._logTime = ''
        self.config_type = config_type
        print("EchoReaderTwo ConfigType: " + str(config_type))
        # Instantiate the configuration file parser
        self.config = RawConfigParser()
        # Make it case-sensitive for parameter names
        self.config.optionxform = str
        # Open the configuration file
        self.config.read ("EchoReader.ini")
        testparam = self.config.getint(self.config_type, 'Test_Param')
        print("TestParam: " + str(testparam))


    ## Opens a DART dump file for reading.
    #  @param filepath A string containing the full path to a dump file.
    #  Read in and bin the first 3 header description lines
    def open_for_reading(self, filepath):
        """ Opens A DARTS dump file for reading.

        The parameter is the full path to a dump file."""
        self._filepath = filepath
        print("EchoReaderTwo Opening %s\n" % filepath)
        try:
            self._fileId = open(filepath, 'r')
        except Exception as e:
            print("Exception thrown opening file: " + str(e))
        else:
            self._filepath = filepath
            for i in range(0, 3):
                foo = self.next_line()


    # Just in case the header lines are different from the tsv file
    def open_csv_file(self, filepath, mode):
        """ Opens A DARTS dump file for reading.

        The parameter is the full path to a dump file."""
        self._filepath = filepath
        #print("Opening %s\n" % filepath)
        try:
            self._fileId = open(filepath, 'r')
            if mode == 'realtime':
                self._fileId.seek(0, 2)
            self._stop = 0
            self._eof = 0
        except Exception as e:
            print("Exception thrown opening file: " + str(e))
        else:
            self._filepath = filepath
            # Can't do this if interactive...
            #for i in range(0, 3):
                #foo = self.next_line()


    def close_input(self):
        """ Closes access to the dump file. """
        if self._fileId == None:
            return
        try:
            self._fileId.close()
            self._stop = 1
        except Exception as e:
            print("Error closing file")


    def stop(self):
        """ Module wide marker to implement the stop reading file function."""
        self._stop = 1

    def get_pointer(self):
        return self._fileId.tell()

    def seek_tolocation(self,location):
        return self._fileId.seek(location)

    def get_line(self):
        return self._line
    
    def next_line(self):
        """ Return the next line from the dump file."""
        self._error = self._stop
#        if self._stop:
#            return self._stop
        try:
            #print("What is going on readline: " + self._fileId.readline())
            self._line = self._fileId.readline().rstrip('\n')	# Works but screws up text field scrolling...
####            print("next_line: " + self._line)
        except Exception as e:
            print("Exception thrown reading file")
            self._line = "ERROR"
            self._error = 100
        finally:
            return self._error

#Events\tentryNo\tseverity\tsource\tworkstation\tobt\tlogTime\tmessage
#TM Parameters\tpk\tname\trawValue\tengValue\tmonState\tgenerationTime\trawValidity\tengValidity\tid\treceivedTime\tbitOffset\tbitSize\tsample\tcurve\tlimits\talert
#Commands\tname\tentryNo\tpktOffset\tpktSize\tid\tapid\tssc\ttype\tstype\tcompleted\ttimeTagged\tstageHistory\tmapid\tobt\ttrx\tlastUpdateTime\tparams

    def decode_line(self):
        """ Return a string built from decoding the input line from the dump file."""
        search_pattern = re.compile("^TM Parameters")
        words = self._line.split('\t')
        if search_pattern.search(self._line):
            # TM, Generation time, name, rawValue, engValue, monState
            return (LineType.TM, (words[6],words[2],words[3],words[4],words[5]))
        search_pattern = re.compile("^Events")
        if search_pattern.search(self._line):
            # Event, Generation time, log time, entryNo, severity, source, workstation, message...
            return (LineType.Event, words[6] + " Evt: " + words[1] + "/" + words[2] + " " + words[3] + " " +  words[4] + " ".join(words[7:]))
        search_pattern = re.compile("^Commands")
        if search_pattern.search(self._line):
            type_subtype = self.decode_tc_type_subtype(words[8], words[9])
            # Command, lastupdateTime, ssc?
            return (LineType.TC, words[15] + " TC: " + words[6] + " (0x" + str(int(words[6], base=16)) + ") " + type_subtype)

        self._eof = 1
        return (LineType.Err, "Unknown line")


# TM,trx,id,spid,name,apid,ssc,type,stype,pi1,vc,obt,obtOk,crcFlag,crcOk,alarm,warning
# TMPAR,name,rawValue,engValue,monState,sample,packetRef
# EV,logTime,entryNo,severity,source,workstation,message

    def decode_csv_line(self):
        """ Return a string built from decoding the input line from the dump file."""
        reject_pattern  = re.compile("name")    # TM, TMPAR
        reject_pattern1 = re.compile("logTime") # EV
        search_pattern = re.compile("^TMPAR")
        event_pattern = re.compile("^EV")
        packet_pattern = re.compile("^TM")

        words = self._line.split(',')
        
        if (reject_pattern.search(self._line)) or (reject_pattern1.search(self._line)):
            #self._eof = 1
            print("REJECT")
            return (LineType.Err, "Unknown line")

        if search_pattern.search(self._line):
            #print('TMPAR')
            # TM, name (mib id), rawValue, engValue, monState
            #TMPAR_NAME
            tmpar_name = self.config.getint(self.config_type, 'TMPAR_NAME')
            tmpar_rawvalue = self.config.getint(self.config_type, 'TMPAR_RAWVALUE')
            tmpar_engvalue = self.config.getint(self.config_type, 'TMPAR_ENGVALUE')
            tmpar_state = self.config.getint(self.config_type, 'TMPAR_STATE')
            tmpar_obt = self.config.getint(self.config_type, 'TMPAR_OBT')
            logTime = '0'
            #use log time that came from TM,
            if(len(self._logTime) > 0) :
                logTime = self._logTime

            #If it is available and the TMPAR has a OBT time then use it as default
            if tmpar_obt > -1 :
                logTime = words[tmpar_obt]
                
            return (LineType.TM, (logTime, words[tmpar_name],words[tmpar_rawvalue],words[tmpar_engvalue],words[tmpar_state]))
        
        search_pattern = re.compile("^EV")
        if search_pattern.search(self._line):
            #print('decode_csv_line: EV')
            # Event, logTime, entryNo, severity, source, workstation, message
            ev_obt = self.config.getint(self.config_type, 'EV_OBT')
            ev_entryno = self.config.getint(self.config_type, 'EV_ENTRYNO')
            ev_severity = self.config.getint(self.config_type, 'EV_SEVERITY')
            ev_source = self.config.getint(self.config_type, 'EV_SOURCE')
            ev_workstation = self.config.getint(self.config_type, 'EV_WORKSTATION')
            ev_message = self.config.getint(self.config_type, 'EV_MESSAGE')
            ##if no log time from TM, then lets set it to the event time.
            #the next TM, will overwrite it.

            return (LineType.Event, words[ev_obt] + " Evt: " + words[ev_entryno] + "/" + words[ev_severity] + " " + words[ev_source] + " " +  words[ev_workstation] + " ".join(words[ev_message]))
        
        search_pattern = re.compile("^Commands")
        if search_pattern.search(self._line):
            #print('COMMAND')
            type_subtype = self.decode_tc_type_subtype(words[8], words[9]) 
            return (LineType.TC, words[15] + " TC: " + words[6] + " (0x" + hex(words[6]) + ") " + type_subtype)
        
        search_pattern = re.compile("^TM")
        if search_pattern.search(self._line):

            tm_obt = self.config.getint(self.config_type, 'TM_OBT')
            tm_apid = self.config.getint(self.config_type, 'TM_APID')
            tm_apid2 = self.config.getint(self.config_type, 'TM_APID2')
            tm_type = self.config.getint(self.config_type, 'TM_TYPE')
            tm_subtype = self.config.getint(self.config_type, 'TM_SUBTYPE')
            tm_warn = self.config.getint(self.config_type, 'TM_WARN')
            tm_logtime = self.config.getint(self.config_type, 'TM_LOGTIME')
            type_subtype = self.decode_tm_type_subtype(words[tm_type], words[tm_subtype]) 
            self._logTime = words[tm_logtime]
            return (LineType.TM_PACKET, (self._logTime, words[tm_apid], type_subtype, words[tm_warn]))
        
        self._eof = 1
        #return (LineType.Err, "Unknown line")
        return (LineType.Empty, "Unknown line")


    def data_available(self):
        """ Return true when there is more data available in the dump file."""
        #print("Data available: " + str(self._eof) + ", " + str(self._stop))
        return (self._eof == 0) and (self._stop == 0)


    def decode_tc_type_subtype(self, t, s):
        try:
            return Euclid_TC_codes.Euclid_tc_types[(int(t), int(s))]
        except Exception as e:
            #print("EXCEPTION!")
            return "Unknown type, subtype (" + t + ", " + s + ")"


    def decode_tm_type_subtype(self, t, s):
        try:
            return Euclid_TM_codes.Euclid_tm_types[(int(t), int(s))]
        except Exception as e:
            return "Unknown type, subtype (" + t + ", " + s + ")"


