""" 
This class creates and laysout a page of HK parameters and associated plot to be displayed in a tab.
"""

import sys
import re

from PyQt5 import QtWidgets, QtCore, QtGui      #, uic, QtQml, QtQuick

from PyQt5.QtGui import QPainter, QBrush, QPen
from PyQt5.QtCore import Qt

from random import randint
from collections import deque

import LabelFrame
import iws_defs

"""
[{'mib_name': 'VIS_ROE_CCD1_TTO01', 'mib_id': 'VS1T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO02', 'mib_id': 'VS2T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO03', 'mib_id': 'VS3T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO04', 'mib_id': 'VS4T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO05', 'mib_id': 'VS5T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO06', 'mib_id': 'VS6T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO07', 'mib_id': 'VS7T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO08', 'mib_id': 'VS8T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO09', 'mib_id': 'VS9T2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO10', 'mib_id': 'VSAT2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO11', 'mib_id': 'VSBT2020', 'source': 'rt_slow', 'unit': 'C'}, {'mib_name': 'VIS_ROE_CCD1_TTO12', 'mib_id': 'VSCT2020', 'source': 'rt_slow', 'unit': 'C'}]
"""

fpn = re.compile('[+-]?(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?')

class GroupFrame(QtWidgets.QFrame):

    def __init__(self, json_data):
        super(GroupFrame, self).__init__()

        # Dictionary of LabelFrames, key is "mib_id"
        self.labels = {}

        layout = QtWidgets.QVBoxLayout()
        layout.setSpacing(0)
        layout.setContentsMargins(0, 0, 0, 0)

 #       print(f"json_data length = {len(json_data)}")
        if len(json_data) == 0:
#            pass
            label = LabelFrame.LabelFrame("")
            layout.addWidget(label)
        else:
            for i in json_data:
                label = LabelFrame.LabelFrame(i)
                layout.addWidget(label)
                
                mib_id = i['mib_id']
                self.labels[mib_id] = label
                #self.labels[mib_id] = (label, None)

        layout.addStretch(1)   # Prevents the labels from spacing out if not enough to fill entire frame
        self.setLayout(layout)

        self.populated = len(self.labels) != 0
        self.selected = False

        #print(f"GroupFrame {self.populated}")
        #self.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Raised)


    def debug(self):
        for label in self.labels:
            self.labels[label].debug()


    def reset_selected(self):
        """
        Return a list of random ingredients as strings.

        :param kind: Optional "kind" of ingredients.
        :type kind: list[str] or None
        :raise lumache.InvalidKindError: If the kind is invalid.
        :return: The ingredients list.
        :rtype: list[str]
        
        """ 
        self.selected = False
        self.setLineWidth(0)
        self.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Plain)


    def parameters(self):
        return self.labels


    def eng_val(self, mib_name):
        return self.labels[mib_name].eng_val


    def get_labels(self):
        l = []
        for i in self.labels.keys():
            print(f"GroupFrame: {i} -> {self.labels[i].get_label_frame()}")
            l.append(self.labels[i].get_label_frame())
        #return self.labels
        return l


    def display_mib_name(self):
        for f in self.labels:
            self.labels[f].display_name()


    def display_mib_id(self):
        for f in self.labels:
            self.labels[f].display_mib_id()

    def display_engineering(self):
        for f in self.labels:
            self.labels[f].display_engineering()


    def display_raw(self):
        for f in self.labels:
            self.labels[f].display_raw()


    # Return 0 ok, 1 warn, 2 ool
    def update_tm(self, parameter_string):
        try:
            #mib_id = parameter_string[1]
            mib_id = parameter_string[iws_defs.TM_MIB_ID_INDEX]
            label_frame = self.labels[mib_id]

            #raw_val_str = parameter_string[2]
            #eng_val_str = parameter_string[3]
            #status      = parameter_string[4]
            raw_val_str = parameter_string[iws_defs.TM_RAW_VAL_INDEX]
            eng_val_str = parameter_string[iws_defs.TM_ENG_VAL_INDEX]
            status      = parameter_string[iws_defs.TM_STATUS_INDEX]

            raw_val = float(raw_val_str)

            if fpn.match(eng_val_str):
                eng_val = float("%3.3f" % float(eng_val_str))
                eng_val_str = str(eng_val)
            else:
                eng_val = raw_val

            if mib_id == "VOGT2856":
                print(f"GroupFrame {mid_id} seen")

            return label_frame.update_display1(raw_val, eng_val, raw_val_str, eng_val_str, status)
        except Exception as e:
            return 0
            pass


    def mousePressEvent(self, event):
        print(f"Mouse press event: {event}")


    def mouseReleaseEvent(self, event):
        print(f"Mouse release event: {event.button()}")
        if event.button() == Qt.RightButton:
            print("Right button")
            #self.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Raised)
            if self.selected:
#                self.setStyleSheet("background-color:red;")
                self.setLineWidth(0)
                self.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Plain)
                #self.setStyleSheet("QFrame" "{" "border : 1 px gray;" "}")
                self.selected = False
            else:
                if self.populated:
#                    self.setStyleSheet("background-color:yellow;")
                    self.setLineWidth(5)
                    self.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Raised)
                    self.selected = True
#                    self.debug()
                    #self.setStyleSheet("border : 5 px black;")
