# -*- coding: utf-8 -*-

## @package IWS_MainWindow
# A module to set up the gui.
# Copied from vertical.py.

from PyQt5 import QtCore, QtGui, QtWidgets

import sys
import os
import time
import math

import json

from matplotlib import pyplot
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg

#import matplotlib
#import matplotlib.figure
#from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
#from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar

import numpy as np

#from Euclid import iws_defs, LabelsPage, GraphPage
import iws_defs, LabelsPage, GraphPage

STATUSBAR_HEIGHT = 28

XTOP = 0
YTOP = 0
WIDTH = 1900
HEIGHT = 1130 #1035
GAP = 5

QUARTER_WIDTH       = math.floor(WIDTH / 4)
THREE_QUARTER_WIDTH = QUARTER_WIDTH * 3

TOP_FRAME_X_INSET = GAP
TOP_FRAME_Y_INSET = GAP
TOP_FRAME_WIDTH   = WIDTH - (2 * TOP_FRAME_X_INSET)
TOP_FRAME_HEIGHT  = 48

MIDDLE_FRAME_X_INSET = GAP
MIDDLE_FRAME_Y_INSET = TOP_FRAME_Y_INSET + TOP_FRAME_HEIGHT + GAP #GAP
MIDDLE_FRAME_WIDTH   = WIDTH - (2 * TOP_FRAME_X_INSET)
#MIDDLE_FRAME_HEIGHT  = 1014 - STATUSBAR_HEIGHT #919 - STATUSBAR_HEIGHT #48
MIDDLE_FRAME_HEIGHT  = (HEIGHT - 116) - STATUSBAR_HEIGHT #919 - STATUSBAR_HEIGHT #48

LOG_FRAME_X_INSET = 0 #GAP # 0
LOG_FRAME_Y_INSET = 0 #TOP_FRAME_Y_INSET + TOP_FRAME_HEIGHT + GAP # 0
LOG_FRAME_WIDTH   = QUARTER_WIDTH
LOG_FRAME_HEIGHT  = MIDDLE_FRAME_HEIGHT #919 - STATUSBAR_HEIGHT

DISPLAY_FRAME_X_INSET = LOG_FRAME_X_INSET + LOG_FRAME_WIDTH + GAP
DISPLAY_FRAME_Y_INSET = 0 #TOP_FRAME_Y_INSET + TOP_FRAME_HEIGHT + GAP
DISPLAY_FRAME_WIDTH   = THREE_QUARTER_WIDTH - (3 * GAP)
DISPLAY_FRAME_HEIGHT  = MIDDLE_FRAME_HEIGHT #919 - STATUSBAR_HEIGHT

BOTTOM_FRAME_X_INSET = GAP
BOTTOM_FRAME_Y_INSET = LOG_FRAME_Y_INSET + LOG_FRAME_HEIGHT + GAP
BOTTOM_FRAME_Y_INSET = MIDDLE_FRAME_Y_INSET + MIDDLE_FRAME_HEIGHT + GAP
BOTTOM_FRAME_WIDTH   = WIDTH - (2 * GAP)
BOTTOM_FRAME_HEIGHT  = 48

class IWS_MainWindow(object):
#class IWS_MainWindow(object, config_file):

    def __init__(self):
        super(IWS_MainWindow, self).__init__()
        self.parameters = {}

        self.label_pages = []
        self.graph_pages = []


    def update_tm(self, fields):
        for label_page in self.label_pages:
            label_page.update_tm(fields)
        for graph_page in self.graph_pages:
            graph_page.update_tm(fields)


    def update_tm_csv(self, fields):
        for label_page in self.label_pages:
            label_page.update_tm_csv(fields)
        for graph_page in self.graph_pages:
            graph_page.update_tm_csv(fields)


    def update_plots(self):
        #print("update_plots")
        for graph_page in self.graph_pages:
            graph_page.update_plot()


    def create_top_frame(self, parent_frame):
        top_frame = QtWidgets.QFrame(parent_frame)
        top_frame.setGeometry(QtCore.QRect(TOP_FRAME_X_INSET, TOP_FRAME_Y_INSET, TOP_FRAME_WIDTH, TOP_FRAME_HEIGHT))
        top_frame.setEnabled(True)
        #sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        #sizePolicy.setHorizontalStretch(0)
        #sizePolicy.setVerticalStretch(0)
        #sizePolicy.setHeightForWidth(self.top_frame.sizePolicy().hasHeightForWidth())
        #self.top_frame.setSizePolicy(sizePolicy)
        top_frame.setBaseSize(QtCore.QSize(0, 0))
        top_frame.setAutoFillBackground(False)
        top_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        top_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        top_frame.setObjectName("top_frame")
        #####top_frame.setStyleSheet("background-color:green;")

        self.pushButton_start = QtWidgets.QPushButton(top_frame)
        #self.pushButton_start.setGeometry(QtCore.QRect(10, 10, 113, 32))
        self.pushButton_start.setObjectName("pushButton_connect")
        self.pushButton_start.setText("Connect")

        self.pushButton_stop = QtWidgets.QPushButton(top_frame)
        #self.pushButton_stop.setGeometry(QtCore.QRect(1760, 10, 113, 32))
        self.pushButton_stop.setObjectName("pushButton_stop")
        self.pushButton_stop.setText("Stop")

        self.pushButton_quit = QtWidgets.QPushButton(top_frame)
        self.pushButton_quit.setObjectName("pushButton_quit")
        self.pushButton_quit.setText("Quit")

        self.file_textbox = QtWidgets.QLabel()
        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(self.pushButton_start)
        layout.addWidget(self.pushButton_stop)
        layout.addWidget(self.file_textbox)

        layout.addStretch(1)
        layout.addWidget(self.pushButton_quit)
        top_frame.setLayout(layout)

        return top_frame

    
    def create_log_frame(self, parent_frame):
        log_frame = QtWidgets.QFrame(parent_frame)
        log_frame.setGeometry(QtCore.QRect(LOG_FRAME_X_INSET, LOG_FRAME_Y_INSET, LOG_FRAME_WIDTH, LOG_FRAME_HEIGHT))
        log_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        log_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        log_frame.setObjectName("log_frame")
        #####log_frame.setStyleSheet("background-color:yellow;")

        event_frame = QtWidgets.QFrame(log_frame)
        event_frame.setGeometry(QtCore.QRect(0, 0, LOG_FRAME_WIDTH, LOG_FRAME_HEIGHT / 2))
        event_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        event_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        event_frame.setObjectName("event_frame")
        #####event_frame.setStyleSheet("background-color:lightgreen;")

        label = QtWidgets.QLabel(event_frame)
        label.setGeometry(QtCore.QRect(0, 0, LOG_FRAME_WIDTH, 16))
        label.setText("Events:")
        label.setObjectName("label")

        self.eventText = QtWidgets.QPlainTextEdit(event_frame)
        self.eventText.setGeometry(QtCore.QRect(0, 16, LOG_FRAME_WIDTH, (LOG_FRAME_HEIGHT / 2) - 16))
        font = QtGui.QFont()

        font.setPointSize(9)
        self.eventText.setFont(font)
        self.eventText.setReadOnly(True)
        self.eventText.setObjectName("eventText")

        tc_frame = QtWidgets.QFrame(log_frame)
        tc_frame.setGeometry(QtCore.QRect(0, LOG_FRAME_HEIGHT / 2, LOG_FRAME_WIDTH, LOG_FRAME_HEIGHT / 2))
        tc_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        tc_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        tc_frame.setObjectName("event_frame")
        #tc_frame.setStyleSheet("background-color:brown;")
        label = QtWidgets.QLabel(tc_frame)
        label.setGeometry(QtCore.QRect(0, 0, LOG_FRAME_WIDTH, 16))
        #label.setText("Telecommands:")
        label.setText("Packets:")
        label.setObjectName("label")
                
        self.tcText = QtWidgets.QPlainTextEdit(tc_frame)
        self.tcText.setGeometry(QtCore.QRect(0, 16, LOG_FRAME_WIDTH, (LOG_FRAME_HEIGHT / 2) - 16))
        font.setPointSize(9)
        self.tcText.setFont(font)
        self.tcText.setReadOnly(True)
        self.tcText.setObjectName("tcText")
        
        return log_frame

    
    def create_graph_frame(self, parent_frame, json_data):
        """ Create a page of graphs. json_data is
        [{name:name,"naxes"  : 1,
			     "yaxis1" : {
			     	      "label" : "Temperature (C)",
				      "min"   : -50,
				      "max"   :  50
			     },
			     "yaxis2" : {

			     },
                             labels:[{mib_name:mn,mib_id:mi,source:src,unit:v},{},...,{}]}, {}, ..., {}] """


        graph_frame = QtWidgets.QFrame(parent_frame)
        graph_frame.setGeometry(QtCore.QRect(0, 0, DISPLAY_FRAME_WIDTH, DISPLAY_FRAME_HEIGHT / 2))
        graph_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        graph_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        graph_frame.setObjectName("graph_frame")
        #####graph_frame.setStyleSheet("background-color:blue;")
        
        self.graph_tabWidget = QtWidgets.QTabWidget(graph_frame)
        self.graph_tabWidget.resize(DISPLAY_FRAME_WIDTH, DISPLAY_FRAME_HEIGHT / 2)
        #self.graph_tabWidget.setGeometry(QtCore.QRect(0, 0, DISPLAY_FRAME_WIDTH, DISPLAY_FRAME_HEIGHT / 2))
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setWeight(75)
        self.graph_tabWidget.setFont(font)
        #self.graph_tabWidget.setStyleSheet("background-color: rgb(252, 1, 7);")
        #####self.graph_tabWidget.setStyleSheet("background-color: red;")
        self.graph_tabWidget.setObjectName("graph_tabWidget")
        #self.graph_tabWidget.setGeometry(QtCore.QRect(0, 0, DISPLAY_FRAME_WIDTH, (DISPLAY_FRAME_HEIGHT / 2) - 10))

        for i in json_data:
            """ i = {name:name,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v} """
            tab_name = i['name']
            tab = GraphPage.GraphPage(graph_frame, i)
            #self.parameters.update(tab.parameters)
            tab.setObjectName(tab_name)
            self.graph_tabWidget.addTab(tab, tab_name)
            self.graph_pages.append(tab)

        return graph_frame


    def create_labels_frame(self, parent_frame, json_data):
        """ Create a page of labels. json_data is
        [{name:name,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v},{},...,{}]}, {}, ..., {}] """
        """
        [{name:name,groups:[{row:r, column:c, labels:[{mib_name:mn,mib_id:mi,source:src,unit:v},{},...,{}]}, {}, ..., {}]}]
        """

        labels_frame = QtWidgets.QFrame(parent_frame)
        labels_frame.setGeometry(QtCore.QRect(0, DISPLAY_FRAME_HEIGHT / 2, DISPLAY_FRAME_WIDTH, DISPLAY_FRAME_HEIGHT / 2))
        labels_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        labels_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        labels_frame.setObjectName("labels_frame")
        #####labels_frame.setStyleSheet("background-color:lightgreen;")
        
        self.labels_tabWidget = QtWidgets.QTabWidget(labels_frame)
        self.labels_tabWidget.resize(DISPLAY_FRAME_WIDTH, DISPLAY_FRAME_HEIGHT / 2)
        #self.labels_tabWidget.setGeometry(QtCore.QRect(0, 0, DISPLAY_FRAME_WIDTH, DISPLAY_FRAME_HEIGHT / 2))
        #####self.labels_tabWidget.setStyleSheet("background-color:blue;")
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setWeight(75)
        self.labels_tabWidget.setFont(font)
        #self.labels_tabWidget.setStyleSheet("background-color: yellow;")
        self.labels_tabWidget.setObjectName("labels_tabWidget")

        for i in json_data:
            """ i = {name:name,groups:[{row:r,column:c,labels:[{mib_name:mn,mib_id:mi,source:src,unit:v}]}]} """
            tab_name = i['name']
            tab = LabelsPage.LabelsPage(labels_frame, i)
            self.parameters.update(tab.parameters)
            tab.setObjectName(tab_name)
            self.labels_tabWidget.addTab(tab, tab_name)
            self.label_pages.append(tab)
        
        return labels_frame


    def create_display_frame(self, parent_frame, json_data):
        """ Create and size the frame which will contain the graph and display tab frames. """
        display_frame = QtWidgets.QFrame(parent_frame)
        display_frame.setGeometry(QtCore.QRect(DISPLAY_FRAME_X_INSET, DISPLAY_FRAME_Y_INSET, DISPLAY_FRAME_WIDTH, DISPLAY_FRAME_HEIGHT))
        display_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        display_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        display_frame.setObjectName("display_frame")
        #display_frame.setStyleSheet("background-color:blue;")

        graph_frame = self.create_graph_frame(display_frame, json_data["graphs"])
        labels_frame = self.create_labels_frame(display_frame, json_data["tabs"])

        return display_frame

    

    def create_bottom_frame(self, parent_frame, json_data):
        bottom_frame = QtWidgets.QFrame(parent_frame)
        bottom_frame.setGeometry(QtCore.QRect(BOTTOM_FRAME_X_INSET, BOTTOM_FRAME_Y_INSET, BOTTOM_FRAME_WIDTH, BOTTOM_FRAME_HEIGHT))
        bottom_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        bottom_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        bottom_frame.setObjectName("bottom_frame")
        #####bottom_frame.setStyleSheet("background-color:red;")

        self.pushButton = QtWidgets.QPushButton(bottom_frame)
        self.pushButton.setGeometry(QtCore.QRect(10, 10, 113, 32))
        self.pushButton.setObjectName("pushButton")
        self.pushButton.setText("PushButton")

        self.pushButton_2 = QtWidgets.QPushButton(bottom_frame)
        self.pushButton_2.setGeometry(QtCore.QRect(1760, 10, 113, 32))
        self.pushButton_2.setObjectName("pushButton_2")
        self.pushButton_2.setText("PushButton2")

        return bottom_frame


    def setupUi(self, MainWindow, config_file):
        #print("Got " + config_file)
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(WIDTH, HEIGHT)

        centralwidget = QtWidgets.QWidget(MainWindow)
        centralwidget.setObjectName("centralwidget")
        centralwidget.setAccessibleName("centralwidget")
        #centralwidget.setStyleSheet("background-color:purple;")

        #self.create_toplevel_frames(self.centralwidget)
        
        main_frame = QtWidgets.QFrame(centralwidget)
        main_frame.setGeometry(QtCore.QRect(XTOP, YTOP, WIDTH, HEIGHT))
        main_frame.setEnabled(True)
        main_frame.setObjectName("main_frame")

        with open(config_file) as f:
            data = json.load(f)

        #print(data)

        middle_frame = QtWidgets.QFrame(main_frame)
        middle_frame.setGeometry(QtCore.QRect(MIDDLE_FRAME_X_INSET, MIDDLE_FRAME_Y_INSET, MIDDLE_FRAME_WIDTH, MIDDLE_FRAME_HEIGHT))
        middle_frame.setEnabled(True)
        middle_frame.setObjectName("middle_frame")
        #middle_frame.setStyleSheet("background-color:purple;")

        top_frame = self.create_top_frame(main_frame)
#        log_frame = self.create_log_frame(main_frame)
#        display_frame = self.create_display_frame(main_frame, data)
        bottom_frame = self.create_bottom_frame(main_frame, data)

#        top_frame = self.create_top_frame(middle_frame)
        log_frame = self.create_log_frame(middle_frame)
        display_frame = self.create_display_frame(middle_frame, data)
#        bottom_frame = self.create_bottom_frame(middle_frame, data)

#        centralwidget.addLayout(self.top_frame)
#        centralwidget.addWidget(self.log_frame)
#        centralwidget.addWidget(self.display_frame)
#        centralwidget.addWidget(self.bottom_frame)
        
        MainWindow.setCentralWidget(centralwidget)

        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1900, 22))
        self.menubar.setObjectName("menubar")

        MainWindow.setMenuBar(self.menubar)

        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        
        MainWindow.setStatusBar(self.statusbar)
        self.statusbar.showMessage('Using configuration: ' + config_file)

        self.retranslateUi(MainWindow)
#        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        #MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        MainWindow.setWindowTitle(_translate("MainWindow", "IWS TC TM Monitor"))
        self.pushButton_2.setText(_translate("MainWindow", "PushButton"))


    def populate_label_tabs(self):
        foo = self.rt_slow_labels_tab
        #foo = self.labels_tab_cdpu
        foo.setupUi()
        #self.rt_slow.setupUi(self)
        self.parameters.update(self.rt_slow_labels_tab.parameters)
        #self.parameters.update(self.labels_tab_cdpu.parameters)
        #print("Button height %d" % self.pushButton.height())


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = IWS_MainWindow()
    ui.setupUi(MainWindow)
    ui.populate_label_tabs()
    #self.rt_slow.setupUi(ui)
    #print("Button height %d" % ui.pushButton.height())
    #print("cw height %s" % MainWindow.centralWidget().accessibleName())
    MainWindow.show()
    sys.exit(app.exec_())
